# ---- [ net_io_tools.py ] -----------------------------------------------------
#
# Tools für den AVR-NET-IO in Python 3.2
#
# liest die 4 ADC-Werte und die 4 Eingänge aus
# Setzt die 8 Ausgangspins und liest deren Status
# Schreibt und löscht Zeilen auf dem LCD
#
# 11.11.12 
#
# -------------------------------------------------------------------------------

import time
import telnetlib

telnet_host    = '192.100.100.200'
telnet_port    = 50290
telnet_timeout = 0.5
netio          = None

# ------------------------------------------------------------------------------
# Ein Kommando senden und die Antwort zurückliefern
# Das Ende der Antwort wird am Linefeed "\n" erkannt

def Send_Cmd(cmd):
    netio.write(cmd.encode('ascii') + b"\n")
    return netio.read_until(b'\n', telnet_timeout)


# ------------------------------------------------------------------------------
# Die Software-Version holen und als Liste zeilenweise zurückgeben
# Die Information besteht aus vier Zeilen, die jeweils mit \r\n abgeschlossen sind
# Daher kann das Ende des gesamten Textes nicht anhand \n erkannt werden.
# Liefert eine Liste mit 4 Zeilen zurück, die 4.Zeile ist leer.

def Get_Version():
    netio.write("VERSION".encode('ascii') + b"\n")
    time.sleep(0.2)             # warten bis die Daten bereitstehen

    t = (netio.read_eager()).decode('ascii')
    return t.split("\r\n")      # in Zeilen aufspalten

# ------------------------------------------------------------------------------
# Einen einzelnen ADC-Wert auslesen
# Bei einem Fehler wird -1 zurückgeliefert
def Get_Adc(channel):
    try:
        v = int(Send_Cmd("GETADC " + str(channel)))
    except:
        v = -1
    return v

# ------------------------------------------------------------------------------
# Alle vier ADC-Werte auslesen und als Liste von Int's zurückliefern
def Get_Adcs():
    l = [0,0,0,0]

    for i in [1,2,3,4]:
        l[i-1] = Get_Adc(i)
        
    return l

# ------------------------------------------------------------------------------
# holt den aktuellen Status der digitalen Ausgänge und liefert eine Liste von Int's

def Get_Port_State():
    l = [0,0,0,0,0,0,0,0]
    t = (Send_Cmd("GETSTATUS")).decode('ascii')  
    t = t[1:10]                  # führendes 'S' und folgendes \r\n' abschneiden

    for i in range(8):
        l[i] = t[7-i]            # die Reihenfolge umkehren
    return l

# ------------------------------------------------------------------------------
# Setzt den Status eines ditalen Ausgangspins auf High oder Low

def Set_Port_State(pin, state):
    Send_Cmd("SETPORT " + str(pin) + '.' + str(state))
    
# ------------------------------------------------------------------------------
# holt den Status der digitalen Eingangspins und lieferte eine Liste von Int's

def Get_Pin_State():
    l = [0,0,0,0]

    for i in [1,2,3,4]:
        t = (Send_Cmd("GETPORT " + str(i))).decode('ascii')
        l[i-1] = int(t[0])           # folgendes \r\n' abschneiden
    return l

# ------------------------------------------------------------------------------
def Start_Telnet():
    global netio
    try:
        netio = telnetlib.Telnet(telnet_host, telnet_port, telnet_timeout)
        version = Get_Version()

        out = "Host: " + str(telnet_host) + " Port: " + str(telnet_port)
        netio.close()
    
    except:
        out = "Error Host: " + str(telnet_host) + " Port: " + str(telnet_port)
# Das leere dritte Feld wird genutzt, um das Fehlen eines NET-IO zu signalisieren
        version = ["Kein AVR-NET-IO", "gefunden !", ""]

    return (out, version)

# ------------------------------------------------------------------------------
def Close_Telnet():
    netio.close()

# ------------------------------------------------------------------------------
def Open_Telnet():
    netio.open(telnet_host, telnet_port)

# --- LCD ----------------------------------------------------------------------
# ------------------------------------------------------------------------------
# Der Server liefert einen Bytestring zurück
# wenn im String an der Position 2:5 der Wert "ACK" zu finden ist,
# dann wurde die Initialisierung des LCD's bestätigt.
# Wird mit "NACK" geantwortet, dann ist vermutlich kein LCD angeschlossen

def Init_LCD():
    ans = (Send_Cmd("INITLCD")).decode('ascii')
    return (ans[:3] == "ACK")

# -------------------------------------------------------------------------------    
# Schreibt eine Zeile, Zeile [1,2], maximale Zeichenzahl 16,
# kürzer Zeichenfolgen werden mit Leerzeichen auf Länge 16 erweitert:
# dadurch wird das Zeilenende automatisch gelöscht.
# Zeichen mit Code > 127 werden durch '$' ersetzt,

def Write_LCD(zeile, textzeile):

    txt = ""
    for i in textzeile:             # Zeichen > 127 durch '$' ersetzen
        if ord(i) < 128:
            txt = txt + i
        else:
            txt = txt + '$'
   
    if len(txt) < 16:               # wenn weniger als 16 Zeichen, 
        txt = txt.ljust(16)         # dann rechts mit ' ' auffüllen
    elif len(txt) > 16:             # wenn länger als 16 Zeichen
        txt = txt[0:16]             # dann auf 16 Zeichen kappen
        
    ans = (Send_Cmd("WRITELCD " + str(zeile) + "." + txt)).decode('ascii')
    return(ans[:3] == "ACK")

# ------------------------------------------------------------------------------
# löscht eine Zeile (wenn der 2. Paramter angegeben wird als Zeile[1,2])
# bei fehlendem 2. Parameter wird das gesamte Display gelöscht

def Clear_LCD(zeile = 0):
    if zeile == 0:
        ans = (Send_Cmd("CLEARLCD")).decode('ascii')
    else:
        ans = (Send_Cmd("CLEARLCD " + str(zeile))).decode('ascii')

    return(ans[:3] == "ACK")

# --- [ eof ] ------------------------------------------------------------------

    

    
